"use strict";
//
// Copyright 2021 Vulcanize, Inc.
//
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Cache = exports.getCache = void 0;
var assert_1 = __importDefault(require("assert"));
var canonical_json_1 = __importDefault(require("canonical-json"));
var ethers_1 = require("ethers");
var level_1 = __importDefault(require("level"));
var fs_extra_1 = __importDefault(require("fs-extra"));
var path_1 = __importDefault(require("path"));
var debug_1 = __importDefault(require("debug"));
var log = debug_1.default('vulcanize:cache');
var getCache = function (config) { return __awaiter(void 0, void 0, void 0, function () {
    var cache, name_1, enabled, deleteOnStart, cacheDirPath;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                if (!config) return [3 /*break*/, 4];
                name_1 = config.name, enabled = config.enabled, deleteOnStart = config.deleteOnStart;
                assert_1.default(name_1);
                cacheDirPath = path_1.default.join(process.cwd(), 'out', name_1 + ".db");
                return [4 /*yield*/, fs_extra_1.default.ensureDir(cacheDirPath)];
            case 1:
                _a.sent();
                if (!deleteOnStart) return [3 /*break*/, 3];
                return [4 /*yield*/, fs_extra_1.default.emptyDir(cacheDirPath)];
            case 2:
                _a.sent();
                _a.label = 3;
            case 3:
                // Enable/disable flag for the cache.
                if (enabled) {
                    cache = new Cache(name_1, cacheDirPath);
                }
                _a.label = 4;
            case 4: return [2 /*return*/, cache];
        }
    });
}); };
exports.getCache = getCache;
var Cache = /** @class */ (function () {
    function Cache(name, dirPath) {
        assert_1.default(name);
        assert_1.default(dirPath);
        this._name = name;
        this._db = level_1.default(dirPath, { valueEncoding: 'json' });
    }
    Cache.prototype.key = function (obj) {
        return this._cacheKey(obj);
    };
    Cache.prototype.get = function (obj) {
        return __awaiter(this, void 0, void 0, function () {
            var key, value, err_1;
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        key = this._cacheKey(obj);
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this._db.get(key)];
                    case 2:
                        value = _a.sent();
                        log(this._name + ": cache hit " + key);
                        return [2 /*return*/, [value, true]];
                    case 3:
                        err_1 = _a.sent();
                        log(this._name + ": cache miss " + key);
                        if (err_1.notFound) {
                            return [2 /*return*/, [undefined, false]];
                        }
                        return [3 /*break*/, 4];
                    case 4: return [2 /*return*/];
                }
            });
        });
    };
    Cache.prototype.put = function (obj, value) {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this._db.put(this._cacheKey(obj), value)];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    Cache.prototype._cacheKey = function (obj) {
        return ethers_1.ethers.utils.keccak256(Buffer.from(canonical_json_1.default(obj)));
    };
    return Cache;
}());
exports.Cache = Cache;
//# sourceMappingURL=cache.js.map